<?php

namespace App\Http\Controllers\Frontend;

use App\Http\Controllers\Controller;
use App\Http\Resources\ProductVariationInfoResource;
use App\Models\AddOnProduct;
use App\Models\Product;
use App\Models\ProductCategory;
use App\Models\ProductTag;
use App\Models\ProductVariation;
use App\Models\Tag;
use App\Models\ProductReview;
use App\Models\Category;
use App\Models\Brand;
use App\Models\Variation;
use App\Models\VariationValue;
use App\Models\ProductVariationCombination;
use Illuminate\Http\Request;

class ProductController extends Controller
{
    # product listing
    public function index(Request $request)
    {    

        $searchKey = null;
        $per_page = 10;
        $sort_by = $request->sort_by ? $request->sort_by : "new";
        $maxRange = Product::max('max_price');
        $min_value = 0;
        $max_value = formatPrice($maxRange, false, false, false, false);

        $products = Product::isPublished();

        # conditional - search by
        if ($request->search != null) {
            $products = $products->where('name', 'like', '%' . $request->search . '%');
            $searchKey = $request->search;
        }
     
        # pagination
        if ($request->per_page != null) {
            $per_page = $request->per_page;
        }

        # sort by
        if ($sort_by == 'new') {
            $products = $products->latest();
        } 

        if ($sort_by == 'best_selling') {
            $products = $products->orderBy('total_sale_count', 'DESC');
        }

        if ($sort_by == 'atoz' || $sort_by == 'ztoa') {

            $products = $products->orderBy('slug', $sort_by == 'atoz' ? 'ASC' : 'DESC');
        }

        if ($sort_by == 'pricemintohigh' || $sort_by == 'pricehightomin') {

            $products = $products->orderBy('min_price', ($sort_by == 'pricemintohigh') ? 'ASC' : 'DESC');
        }

        # by price
        if ($request->min_price != null) {
            $min_value = $request->min_price;
        }
        if ($request->max_price != null) {
            $max_value = $request->max_price;
        }

        if ($request->min_price || $request->max_price) {
            $products = $products->where('min_price', '>=', $min_value)->where('min_price', '<=', $max_value);
        }

        # by category
        if ($request->category_id && $request->category_id != null) {
            $product_category_product_ids = ProductCategory::where('category_id', $request->category_id)->pluck('product_id');
            $products = $products->whereIn('id', $product_category_product_ids);
        }

        # by tag
        if ($request->tag_id && $request->tag_id != null) {
            $product_tag_product_ids = ProductTag::where('tag_id', $request->tag_id)->pluck('product_id');
            $products = $products->whereIn('id', $product_tag_product_ids);
        }
        # conditional

        $tempAttrArray =[];
 
        foreach ($products->get() as $productData) {
            $variationIds = ProductCategory::where('product_id', $productData->id)->pluck('category_id')->toArray();
            $tempAttrArray = array_merge($tempAttrArray, $variationIds);
        }

        // To get unique category_id

        $tempAttrArray = array_unique($tempAttrArray);

        $cateArray = array();

        $cat = Category::whereIn('id', $tempAttrArray)->get();

        foreach ($cat as $pcate) {
            $cateArray[] = $pcate;
            $subcate = Category::where('parent_id', $pcate->categories_id)->get();
            foreach ($subcate as $ccate)
                $cateArray[] = $ccate->categories_id;
        }


        $tempAttrArray = $attriButeArray = [];

        foreach ($products->get() as $productData) {

            $variationIds = ProductVariationCombination::where('product_id', $productData->id)->pluck('variation_id')->toArray();
            $tempAttrArray = array_merge($tempAttrArray, $variationIds);
        }

        // To get unique variation IDs, you can use array_unique
        $tempAttrArray = array_unique($tempAttrArray);

        if (sizeof($tempAttrArray) > 0) {

            foreach ($tempAttrArray as $key => $value) {

                $variationName = Variation::where('id', $value)->first();
                $variationValue = VariationValue::where('variation_id', $value)->get();

                $attriButeArray[$variationName->name] = $variationValue;
            }
        }
        

        $products = $products->paginate(paginationNumber($per_page));

        $tags = Tag::all();


        return getView('pages.product_listing', [
            'products'      => $products,
            'searchKey'     => $searchKey,
            'per_page'      => $per_page,
            'sort_by'       => $sort_by,
            'max_range'     => formatPrice($maxRange, false, false, false, false),
            'min_value'     => $min_value,
            'max_value'     => $max_value,
            'tags'          => $tags,
            'cateArray'     => $cateArray,
            'attriButeArray' => $attriButeArray,
            'selectedAttr'  => $request->attribute ?? [],
        ]);
    }

    # product show
    public function show($slug)
    {   
        $product= Product::with('variations')->where('slug', $slug)->first();
     
        if(empty($product))
         return redirect()->route('home');

        if (auth()->check() && auth()->user()->user_type == "admin") {
            // do nothing
        } else {
            if ($product->is_published == 0) {
                flash(localize('This product is not available'))->info();
                return redirect()->route('home');
            }
        }

        $productCategories              = $product->categories()->pluck('category_id');
        $productIdsWithTheseCategories  = ProductCategory::whereIn('category_id', $productCategories)->where('product_id', '!=', $product->id)->pluck('product_id');

        $relatedProducts                = Product::where('is_published',1)->whereIn('id', $productIdsWithTheseCategories)->get();

        $product_page_widgets = [];
        if (getSetting('product_page_widgets') != null) {
            $product_page_widgets = json_decode(getSetting('product_page_widgets'));
        }   

        $productReview = ProductReview::where('product_id' , $product->id)->where('status' , 1)->get();
       
        return getView('pages.product_detail', ['product' => $product, 'relatedProducts' => $relatedProducts, 'product_page_widgets' => $product_page_widgets , 'productReview' => $productReview]);
    }

    # product info
    public function showInfo(Request $request)
    {
        $product = Product::find($request->id);
        return getView('partials.product_view_box', ['product' => $product]);
    }

    # product variation info
    public function getVariationInfo(Request $request)
    {    
        $price=0;
        $product=Product::where('id',$request->product_id)->first();

        if(!empty($product)) {

            $variationIds=$request->ids;

            $variation_key='';

            if(!empty($variationIds)) {
                foreach($variationIds as $variationKey) {
                   $variation_key.=$variationKey;
                }

                $productVariation = ProductVariation::where('variation_key', $variation_key)->where('product_id', $request->product_id)->first();
              
                $price+= !empty($productVariation) ? $productVariation->price : 0;
            }
            
        
            $discount_applicable = false;
    
            if ($product->discount_start_date == null || $product->discount_end_date == null) {
                $discount_applicable = false;
            } elseif (
                strtotime(date('d-m-Y H:i:s')) >= $product->discount_start_date &&
                strtotime(date('d-m-Y H:i:s')) <= $product->discount_end_date
            ) {
                $discount_applicable = true;
            }
    
            if ($discount_applicable) {
                if ($product->discount_type == 'percent') {
                    $price -= ($price * $product->discount_value) / 100;
                } elseif ($product->discount_type == 'flat') {
                    $price -= $product->discount_value;
                }
            }
    
            foreach ($product->taxes as $product_tax) {
                if ($product_tax->tax_type == 'percent') {
                    $price += ($price * $product_tax->tax_value) / 100;
                } elseif ($product_tax->tax_type == 'flat') {
                    $price += $product_tax->tax_value;
                }
            }
    
            return response()->json(['price' =>formatPrice($price), 'status' => 200]);
        }
    }

    public function quickviewModal(Request $request){ 
       
        $product = Product::where('id' , $request->id)->first();
         
        $returnHTML = view('frontend.partials.quick_view', compact('product'))->render();

        return response()->json($returnHTML);
    }
}
